import logging
from dataclasses import dataclass
from enum import Enum
from typing import List

from click.utils import R

from logninja.json_formatter import JSONFormatter


class LogConsoleType(Enum):
    SIMPLE = "simple_console"
    RICH = "rich_console"
    JSON = "json"


@dataclass
class LogFileConfig:
    filename: str = "logs.jsonl"
    level: int = logging.INFO
    clear_file_on_setup: bool = False
    fmt: logging.Formatter = JSONFormatter()


@dataclass
class LogConsoleConfig:
    level: int = logging.INFO
    log_console_type: LogConsoleType = LogConsoleType.SIMPLE
    extra_vars: List[str] = None



import datetime as dt
import json
import logging
import unittest

from logninja.json_formatter import JSONFormatter


class JSONFormatterTests(unittest.TestCase):

    def test_format(self):
        formatter = JSONFormatter()
        record = logging.LogRecord(
            name="test_logger",
            level=logging.INFO,
            pathname="/path/to/file.py",
            lineno=10,
            msg="Test message",
            args=(),
            exc_info=None,
            func="test_function",
            sinfo=None,
        )

        formatted_message = formatter.format(record)
        self.assertIsInstance(formatted_message, str)
        expected_result = {
            "level": "INFO",
            "message": "Test message",
            "timestamp": dt.datetime.fromtimestamp(
                record.created, tz=dt.timezone.utc
            ).isoformat(),
            "logger": "test_logger",
            "module": "file",
            "function": "test_function",
            "line": 10,
            "thread_name": "MainThread",
        }
        self.assertDictEqual(
            expected_result,
            json.loads(formatted_message),
        )


if __name__ == "__main__":
    unittest.main()



import logging
from contextvars import ContextVar
from typing import List

from logninja.configs import LogConsoleConfig, LogConsoleType, LogFileConfig
from logninja.console_formatter import ConsoleFormatter
from logninja.contextvars_filter import ContextVarsFilter
from logninja.json_formatter import JSONFormatter
from logninja.ninja_rich_handler import NinjaRichHandler

logger = logging.getLogger("logninja")


def setup_logging(
    level: int = logging.INFO,
    fmt: logging.Formatter = JSONFormatter(),
    contextvars: List[ContextVar] = [],
    log_file_config: LogFileConfig = None,
    log_console_config: LogConsoleConfig = LogConsoleConfig(),
) -> None:
    root_handler = get_root_handler(log_console_config=log_console_config)
    root_handler.addFilter(ContextVarsFilter(contextvars=contextvars))

    root_logger = logging.getLogger()
    root_logger.setLevel(log_console_config.level)
    root_logger.addHandler(root_handler)

    if log_file_config:
        if log_file_config.clear_file_on_setup:
            with open(log_file_config.filename, "w") as f:
                f.write("")
        file_handler = logging.FileHandler(log_file_config.filename)
        file_handler.setFormatter(fmt=log_file_config.fmt)
        file_handler.addFilter(ContextVarsFilter(contextvars=contextvars))
        file_handler.setLevel(log_file_config.level)
        root_logger.addHandler(file_handler)

    logger.info("Logging setup complete", extra=dict(users="adfdfs"))


def get_root_handler(log_console_config: LogConsoleConfig):
    log_console_type = log_console_config.log_console_type
    if log_console_type == LogConsoleType.SIMPLE:
        stream_handler = logging.StreamHandler()
        stream_handler.setFormatter(fmt=ConsoleFormatter)
        return stream_handler
    if log_console_type == LogConsoleType.RICH:
        ninja_rich_handler = NinjaRichHandler(extras=log_console_config.extra_vars)
        ninja_rich_handler.setFormatter(logging.Formatter(fmt="%(message)s"))
        return ninja_rich_handler
    if log_console_type == LogConsoleType.JSON:
        json_handler = logging.StreamHandler()
        json_handler.setFormatter(fmt=JSONFormatter())
        return json_handler
    raise ValueError("Invalid log_console_type")



import logging
import time
from contextvars import ContextVar
from datetime import datetime
from os import times
from typing import List


class NinjaRichHandler(logging.Handler):
    def __init__(self, extras: List[str] = [], level: int = logging.INFO):
        try:
            from rich import get_console
        except ImportError:
            raise ImportError(
                "Rich is required for rich logging. Please install it using `pip install logninja[rich]`"
            )
        super().__init__(level=level)
        self.extras = extras
        self.console = get_console()

    def emit(self, record: logging.LogRecord):
        message = self._get_message(record)
        time_str = self._format_time(record)
        level_color = self._get_level_color(record.levelno)
        extras = self.get_extras(record)
        extras_str = self._format_extras(extras)
        self.console.print(
            f"[grey7]\[{time_str}][/grey7][orange4]\[{record.name}][/orange4][{level_color}]\[{record.levelname}][/{level_color}]    {message}{extras_str}"
        )

    def _get_level_color(self, level: int):
        if level >= logging.CRITICAL:
            return "bold red"
        if level >= logging.ERROR:
            return "bold red"
        if level >= logging.WARNING:
            return "yellow"
        if level >= logging.INFO:
            return "green"
        if level >= logging.DEBUG:
            return "blue"
        return "white"

    def _format_time(self, record: logging.LogRecord, fmt: str = "%Y-%m-%d %H:%M:%S"):
        timestamp = record.created
        dt_object = datetime.fromtimestamp(timestamp)
        time_str = dt_object.strftime(fmt)
        return time_str

    def get_extras(self, record: logging.LogRecord):
        extras = {}
        for extra in self.extras:
            if hasattr(record, extra):
                extras[extra] = getattr(record, extra)
        return extras

    def _format_extras(self, extras: dict):
        extras_str = "    "
        for key, value in extras.items():
            extras_str += f"[bold italic yellow]\[{key}:[/bold italic yellow] [italic]{value}[/italic][bold italic yellow]][/bold italic yellow]"
        return extras_str

    def _get_message(self, record: logging.LogRecord):
        message = record.getMessage()
        if self.formatter:
            record.message = record.getMessage()
            formatter = self.formatter
            if hasattr(formatter, "usesTime") and formatter.usesTime():
                record.asctime = formatter.formatTime(record, formatter.datefmt)
            message = formatter.formatMessage(record)
        return message



import asyncio
import functools
import logging
import time

from logninja import logger as logninja_logger


def log_execution(logger: logging.Logger = logninja_logger):
    """
    Decorator to log the execution time of a function or coroutine.

    Args:
        logger (logging.Logger): The logger to be used.

    Returns:
        Callable: The decorator.
    """

    def decorator(func):
        def log_start(*args, **kwargs):
            params = {"args": list(args), "kwargs": kwargs}
            logger.info(
                f"Starting execution of function {func.__qualname__}",
                extra={"params": params},
            )

        def log_end(end: float, start: float):
            logger.info(f"{func.__qualname__} executed in {end - start:.4f} seconds")

        @functools.wraps(func)
        def sync_wrapper(*args, **kwargs):
            log_start(*args, **kwargs)

            start = time.time()
            result = func(*args, **kwargs)
            end = time.time()

            log_end(end, start)
            return result

        async def async_wrapper(*args, **kwargs):
            log_start(*args, **kwargs)

            start = time.time()
            result = await func(*args, **kwargs)
            end = time.time()

            log_end(end, start)
            return result

        return async_wrapper if asyncio.iscoroutinefunction(func) else sync_wrapper

    return decorator
